/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "common.h"

void shaderCommonInit(shadercommon_t *shader, char *vert, char *frag) {
  shaderInit(&shader->shader, vert, frag);

  shader->uniformProjection = shaderGetUniform(
    &shader->shader, SHADER_COMMON_UNIFORM_PROJECTION
  );
  shader->uniformView = shaderGetUniform(
    &shader->shader, SHADER_COMMON_UNIFORM_VIEW
  );
  shader->uniformModel = shaderGetUniform(
    &shader->shader, SHADER_COMMON_UNIFORM_MODEL
  );
  shader->uniformColor = shaderGetUniform(
    &shader->shader, SHADER_COMMON_UNIFORM_COLOR
  );
  shader->uniformTexture = shaderGetUniform(
    &shader->shader, SHADER_COMMON_UNIFORM_TEXTURE
  );
}

void shaderCommonUseCamera(shadercommon_t *shader, camera_t *camera) {
  shaderUseCamera(
    &shader->shader, shader->uniformView, shader->uniformProjection, camera
  );
}

void shaderCommonUseTexture(shadercommon_t *shader, texture_t *texture) {
  shaderUseTexture(&shader->shader, shader->uniformTexture, texture);
}

void shaderCommonUseColor(shadercommon_t *shader, pixel_t color) {
  shaderUseColor(&shader->shader, shader->uniformColor, color);
}

void shaderCommonUsePosition(shadercommon_t *shader,
  float x, float y, float z,
  float pitch, float yaw, float roll
) {
  shaderUsePosition(
    &shader->shader, shader->uniformModel, x, y, z, pitch, yaw, roll
  );
}

void shaderCommonUsePositionAndScale(
  shadercommon_t *shader,
  float x, float y, float z,
  float pitch, float yaw, float roll,
  float scaleX, float scaleY, float scaleZ
) {
  shaderUsePositionAndScale(
    &shader->shader, shader->uniformView,
    x, y, z, pitch, yaw, roll, scaleX, scaleY, scaleZ
  );
}