/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#pragma once
#include "../../libs.h"
#include "../../scene/scene.h"
#include "../file/assetmanager.h"
#include "scenes/gamescene.h"

typedef struct {
  /** Game Engine Instance */
  engine_t engine;

  /** Main game scene */
  pokergamescene_t gameScene;
} pokergame_t;

/**
 * Initializes the game state for the poker game.
 * 
 * @param game Game to initialize.
 * @returns True if successful, otherwise false.
 */
bool pokerGameInit(pokergame_t *game);

/**
 * Updates the poker game instance.
 * @param game Poker game to update for.
 */
void pokerGameUpdate(pokergame_t *game);

/**
 * Disposes a previously initialized poker game instance.
 * @param game Game to dispose.
 */
void pokerGameDispose(pokergame_t *game);