/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "gamescene.h"

void _pokerGameSceneLoaded(scene_t *scene) {
  pokergamescene_t *gameScene;
  char *uniforms[4] = { "u_View", "u_Proj", "u_Model", "u_Text" };

  gameScene = (pokergamescene_t *)scene->user;


  shaderGetUniformArray(
    &gameScene->shaderWorld->data.shader.shader, gameScene->uniformsWorld,
    uniforms, 4
  );
  shaderGetUniformArray(
    &gameScene->shaderList->data.shader.shader, gameScene->uniformsList,
    uniforms, 4
  );
  
  gameScene->passOnly = renderPassAdd(
    &scene->renderList, &gameScene->shaderWorld->data.shader.shader
  );
}

void _pokerGameSceneRender(scene_t *scene) {
  camera_t camera;
  pokergamescene_t *gameScene;
  renderpass_t *pass;

  gameScene = (pokergamescene_t *)scene->user;
  pass = sceneUsePass(scene, gameScene->passOnly);
  
  cameraPerspective(&camera, 75,
    (float)pass->frame.texture.width / (float)pass->frame.texture.height,
    0.01f, 1000.0f
  );
  cameraLookAt(&camera, 3,3,3, 0,0,0);

  shaderUseCamera(pass->shader,
    gameScene->uniformsWorld[0], gameScene->uniformsWorld[1], &camera
  );
  shaderUsePosition(pass->shader, gameScene->uniformsWorld[2], 
    0,0,0, scene->engine->time.current,scene->engine->time.current,0
  );
  shaderUseTexture(pass->shader, gameScene->uniformsWorld[3],
    &gameScene->textureTest->data.scaledTexture.texture
  );
  primitiveDraw(&gameScene->cube, 0, -1);
}

void pokerGameSceneInit(pokergamescene_t *gameScene, engine_t *engine) {
  sceneInit(&gameScene->scene, engine, (void *)gameScene);
  gameScene->scene.onLoaded = &_pokerGameSceneLoaded;
  gameScene->scene.onRender = &_pokerGameSceneRender;

  // Fonts
  gameScene->font = assetManagerLoadFont(
    &engine->assetManager, gameScene->scene.assetOwner,
    "fonts/opensans/OpenSans-Bold.ttf"
  );

  // Shaders
  gameScene->shaderWorld = assetManagerLoadShader(
    &engine->assetManager, gameScene->scene.assetOwner,
    "shaders/textured.vert", "shaders/textured.frag"
  );

  gameScene->shaderList = assetManagerLoadShader(
    &engine->assetManager, gameScene->scene.assetOwner,
    "shaders/singlerenderlist.vert", "shaders/singlerenderlist.frag"
  );

  // Textures
  gameScene->textureTest = assetManagerLoadScaledTexture(
    &engine->assetManager, gameScene->scene.assetOwner,
    "textures", "test_texture", 0
  );
  // gameScene->textureTest = assetManagerLoadTexture(
  //   &engine->assetManager, gameScene->scene.assetOwner, "textures/test_texture.png"
  // );


  // Cube
  cubeInit(&gameScene->cube, 1, 1, 1);
}

void pokerGameSceneUpdate(pokergamescene_t *gameScene) {
  int i = 3;
  sceneUpdateToBackbuffer(&gameScene->scene,
    &gameScene->shaderList->data.shader.shader, gameScene->uniformsList,
    &gameScene->shaderWorld->data.shader.shader, gameScene->uniformsWorld
  );
}

void pokerGameSceneDispose(pokergamescene_t *gameScene) {

}