/**
 * Copyright (c) 2021 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "game.h"

bool sandboxGameInit(sandboxgame_t *game) {
  uint8_t i;

  // Initialize the scene.
  sceneInit(&game->scene, &game->engine);
  
  // Load Shader
  game->shader = assetManagerLoadShader(
    &game->engine.assetManager, game->scene.assetOwner,
    "shaders/textured.vert", "shaders/textured.frag"
  );
  game->shader2 = assetManagerLoadShader(
    &game->engine.assetManager, game->scene.assetOwner,
    "shaders/test.vert", "shaders/test.frag"
  );

  // Load the texture
  game->scaledTexture = assetManagerLoadScaledTexture(
    &game->engine.assetManager, game->scene.assetOwner,
    "textures", "test_texture"
  );
  game->textureScale = assetManagerLoadTextureScale( 
    &game->engine.assetManager, game->scene.assetOwner,
    &game->scaledTexture->data.scaledTexture.scaledTexture, 0
  );

  // quadInit(&game->quad, 0, 0,0,0,0, 1,1,1,1);
  cubeInit(&game->quad, 1, 1, 1);

  i = renderPassAdd(&game->scene.renderList, &game->shader->data.shader.shader);
  i = renderPassAdd(&game->scene.renderList, &game->shader->data.shader.shader);

  return true;
}

void sandboxGameUpdate(sandboxgame_t *game) {
  camera_t camera;
  float n;
  shader_t *shader, *shader2;

  n = assetManagerProgressGet(&game->engine.assetManager);
  if(n < 1.0f) return;

  sceneRenderStart(&game->scene);

  // First pass
  shader = &game->shader->data.shader.shader;
  shader2 = &game->shader2->data.shader.shader;
  shaderuniform_t uniView = shaderGetUniform(shader, "u_View");
  shaderuniform_t uniProj = shaderGetUniform(shader, "u_Proj");
  shaderuniform_t uniModel = shaderGetUniform(shader, "u_Modl");
  shaderuniform_t uniColor = shaderGetUniform(shader, "u_Colr");
  shaderuniform_t uniTexture = shaderGetUniform(shader, "u_Text");

  renderpass_t *pass = renderListRenderPass(
    &game->scene.renderList, &game->engine, 0
  );
  cameraPerspective(&camera, 45.0f,
    game->engine.render.width/game->engine.render.height, 0.001f, 1000.0f
  );
  cameraLookAt(&camera, 3,3,3, 0,0,0);
  
  shaderUseCamera(pass->shader, uniView, uniProj, &camera);
  shaderUseColor(pass->shader, uniColor, PIXEL_COLOR_WHITE);
  shaderUseTexture(pass->shader, uniTexture, &game->textureScale->data.scaleTexture.texture);
  shaderUsePosition(pass->shader, uniModel, 0,0,0,
    game->engine.time.current/2.0f, game->engine.time.current, 0
  );
  primitiveDraw(&game->quad, 0, -1);

  // // Second Pass
  pass = renderListRenderPass(&game->scene.renderList, &game->engine, 1);
  shaderUsePosition(pass->shader, uniModel, 0,0,0,
    -game->engine.time.current/2.0f, -game->engine.time.current, 0
  );
  primitiveDraw(&game->quad, 0, -1);

  // Merge Shader
  shaderuniform_t uniTextureSet[2];
  uniTextureSet[0] = shaderGetUniform(shader2, "u_Text0");
  uniTextureSet[1] = shaderGetUniform(shader2, "u_Text1");

  sceneRenderEnd(&game->scene, shader2,
    shaderGetUniform(shader2, "u_View"),
    shaderGetUniform(shader2, "u_Proj"),
    shaderGetUniform(shader2, "u_Modl"),
    uniTextureSet,
    shader, uniView, uniProj, uniModel, uniTexture
  );
}

void sandboxGameDispose(sandboxgame_t *game) {
  sceneDispose(&game->scene);
  // assetManagerHolderRelease(&game->engine.assetManager, game->assetOwner);
  // primitiveDispose(&game->quad);
}